/* *************************************************************
   RWhois Software

   Copyright (c) 1994 Scott Williamson and Mark Kosters
   Copyright (c) 1996-2000 Network Solutions, Inc.

   See the file LICENSE for conditions of use and distribution.
**************************************************************** */

#include "ip_network.h"

#include "common_regexps.h"
#include "misc.h"
#include "defines.h"

/* address convert from ascii ip network string to long */
unsigned long
addrstring_to_long(addstr, status)
  char *addstr;
  int  *status;
{
  unsigned long addr;
  int           o[4];
  int           num_octets;
  int           i;
  
  bzero(o, sizeof(o));
  *status = FALSE;
  
  
  /* Check prefix and prefix length */
  num_octets = sscanf(addstr, "%d.%d.%d.%d", &o[0], &o[1], &o[2], &o[3]);

  /* clear unused octets */
  for (i = num_octets; i < 4; i++)
  {
    o[i] = 0;
  }

  /* check the ranges */
  for (i = 0; i < num_octets; i++)
  {
    if (o[i] < 0 || o[i] > 255)
    {
      return 0;
    }
  }
    
  addr = (((unsigned)o[0] & 0xFF) << 24) + (((unsigned)o[1] & 0xFF) << 16) +
         (((unsigned)o[2] & 0xFF) << 8) + ((unsigned)o[3] & 0xFF);

  *status = TRUE;

  return(addr);
}

int *
get_octets_from_long(prefix)
  unsigned long prefix;
{
  static int octets[4];
  int        shift;
  int        i;
  
  bzero(octets, sizeof(octets));

  for (i = 0; i < 4; i++)
  {
    shift = 24 - (i * 8);
    octets[i] = (prefix >> shift) & 0XFF;
  }

  return(octets);
}
    
/* Address Convert from long to str */
char *
long_to_addrstring(t, buf)
  unsigned long t;
  char          *buf;
{
  int   *octs;

  octs = get_octets_from_long(t);
  
  sprintf(buf, "%d.%d.%d.%d", octs[0], octs[1], octs[2], octs[3]);

  return(buf);
}

unsigned long
get_mask_from_len(len)
  int   len;
{
  unsigned long mask = 0xffffffff;
  
  if ((len <= 0) || (len > 32))
  {
    return(0);
  }

  mask = mask << (32 - len);

  return(mask);
}

/* Mask addr to prefix */
unsigned long
mask_addr_to_len(addr, len)
  unsigned long addr;
  int           len;

{
  unsigned long mask = get_mask_from_len(len);
  return(addr & mask);
}

/* compares to numerical IP addresses in a endian independent manner
   (-1 if a < b, 0 if a = 0, 1 if a > b) */
int
compare_addr(a, b)
  unsigned long a;
  unsigned long b;
{
  int   oa[4];
  int   ob[4];
  int   *o;
  int   i;
  
  o = get_octets_from_long(a);
  bcopy(o, oa, sizeof(oa));

  o = get_octets_from_long(b);
  bcopy(o, ob, sizeof(ob));

  for (i = 0; i < 4; i++)
  {
    if (oa[i] < ob[i])
    {
      return(-1);
    }
    if (oa[i] > ob[i])
    {
      return(1);
    }
  }

  return(0);
}
  
int
is_network_valid_for_searching(value)
  char *value;
{
  static regexp *net_prog = NULL;

  if (!net_prog)
  {
    net_prog = regcomp(NETWORK_REGEXP);
  }
  
  if (regexec(net_prog, value))
  {
    return(TRUE);
  }
  else
  {
    return(FALSE);
  }
}

int
is_network_valid_for_index(line)
  char *line;
{
  static regexp *strict_net_prog = NULL;
  unsigned long prefix;
  int           len;
  
  if (!strict_net_prog)
  {
    strict_net_prog = regcomp(STRICT_IP_NET_REGEXP);
  }

  if (regexec(strict_net_prog, line))
  {
    if (get_network_prefix_and_len(line, &prefix, &len))
    {
      return TRUE;
    }
  }

  return FALSE;
}

int
is_cidr_network(value)
  char *value;
{
  static regexp *net_prog = NULL;

  if (!net_prog)
  {
    net_prog = regcomp(STRICT_CIDR_NET_REGEXP);
  }
  
  if (regexec(net_prog, value))
  {
    return(TRUE);
  }
  else
  {
    return(FALSE);
  }
}

/* determine_network_len_from_policy: given a network IP number
   without an explict prefix length, determine the prefix length by
   the first octet and presence of 0 octets.  If the  */
int
determine_network_len_from_policy(addr)
  unsigned long addr;
{
  int *o;
  int len;
  unsigned long mask;
  
  o = get_octets_from_long(addr);

  if (o[0] < 128)
  {
    len = 8;
  }
  else if (o[0] < 192)
  {
    len = 16;
  }
  else if (o[0] < 224)
  {
    len = 24;
  }

  mask = get_mask_from_len(len);

  /* if the caculated host part has anything in it, something is wrong, so
     we will assume that it is a host address */
  if ((addr & mask) != addr)
  {
    len = 32;
  }

  return(len);
}

/* calulates the network length by looking at the number of non-zero
   octets. 128.0.0.0 would yield 8, 128.1.0.0 would yield 16,
   128.0.1.0 would yield 24, and so on */

int
determine_network_len_from_octets(addr)
  unsigned long addr;
{
  int *o;
  int len;
  int i;
  
  o = get_octets_from_long(addr);

  for (i = 0, len = 0; i < 4; i++)
  {
    if (o[i] > 0)
    {
      len = 8 * (i + 1);
    }
  }

  return(len);
}
  
/* get_network_prefix_prefix_len_and_mask: This function parses
   network prefix and prefix length, given a string in quad-octet
   prefix/prefix length format */
int
get_network_prefix_and_len(str, prefix, prefix_len)
  char          *str;
  unsigned long *prefix;
  int           *prefix_len;
{
  char *buf;
  char *p;
  int  len;
  int  o[4];
  int  status = FALSE;
  
  if (NOT_STR_EXISTS(str) || !prefix || !prefix_len)
  {
    return FALSE;
  }

  bzero(o, sizeof(o));
  
  buf = xstrdup(str);
  
  /* Parse prefix and prefix length */ 
  if ((p = strrchr(buf, '/')) != NULL)
  {
    *p = '\0';
    p++;
    len = atoi(p);
  }
  else
  {
    len = 32;
  }

  *prefix = addrstring_to_long(buf, &status);
  free(buf);

  if (!status)
  {
    return FALSE;
  }
  
  /* check the length */
  if (len < 0 || len > 32)
  {
    return(FALSE);
  }
  
  *prefix_len = len;

  /* fix any bit masking problems */
  *prefix = mask_addr_to_len(*prefix, *prefix_len);
  
  return(TRUE);
}

int
write_network(str, prefix, prefix_len)
  char *str;
  unsigned long prefix;
  int prefix_len;
{
  char buf[18];
  
  if (!str)
  {
    return FALSE;
  }

  sprintf(str, "%s/%d", long_to_addrstring(prefix, buf), prefix_len);

  return TRUE;
}


